#pragma once

#include <cppcore/misc/flags.h>

#if cppfs_os == cppfs_os_linux
    #include <sys/stat.h>
#endif

namespace cppfs
{

    enum class file_type
    {
        unknown = 0,
        directory,
        file,
        symbolic_link,
        hard_link,
        block_device,
        char_device,
        named_pipe,
        socket,
    };

    enum class file_permission
    {
        unknown     = 0,

        user_read,
        user_write,
        user_execute,

        group_read,
        group_write,
        group_execute,

        others_read,
        others_write,
        others_execute,
    };

    struct file_permissions
        : public cppcore::shifted_flags<file_permission>
    {
    public:
        using base_type = cppcore::shifted_flags<file_permission>;

    public:
        using base_type::base_type;

    #if cppfs_os == cppfs_os_linux
    public:
        /**
         * @brief Convert file permissions to unix permissions.
         */
        static inline mode_t to_unix(const file_permissions& p_perm);

        /**
         * @brief Convert unix permissions to file permissions.
         */
        static inline file_permissions from_unix(mode_t p_perm);
    #endif
    };

}
